        list p=16f628a
    
; PILio11.asm
; PIL-IO version 1.1 - mini HPIL/GPIO + HPIL/serial interface
; History:
; 29/05/10 creation based on PIL-Box 1
;          dual device implementation
; 21/06/10 debug
; 09/04/12 version 1.1:
;  no brownout
;  LPD, GET implementation
;  SSRQ, FREN for future usage


        #include <P16F628A.INC>
;
; config bits: HS osc, watchdog on, powerup timer on, code protect on,
; brownout disabled, master clear disabled, low voltage prog disabled, data ee protect disabled.

        __config 0x0106   ; 16F628A


;
; 20-6f: bank of 80 bytes
BUF     equ     0x20        ; buffer for IL frame samples (20-5f)
DECFLG  equ     0x60        ; decoding flags
CURIN   equ     0x61        ; current input sample
CPTBIT  equ     0x62        ; bit counter
CPTSAM  equ     0x63        ; sample counter
FRREGH  equ     0x64        ; input/output frame shift registers
FRREGL  equ     0x65
SAVEW   equ     0x6f        ; save W during IT
; 70-7f: commun area 
ILFLG   equ     0x70            ; IL flags 
FRREC   equ     0      ; frame received
FRAV    equ     1      ; frame available  
FLPD    equ     2      ; LPD received
SSRQ    equ     3      ; set SRQ bit in IDY


CPTB    equ     0x71            ; general purpose counter
FRAMEH  equ     0x72            ; IL 11-bit frame in 72-73
FRAMEL  equ     0x73
SBYTE   equ     0x74           ; byte from/to serial
; variables for devices:
ILADDR  equ     0x75     ; IL device base address (bits 0-4). bit 7 means addresses assigned
ILSTAT  equ     0x76     ; IL device state bits
; FLST    FTLK
;    0       0     idle
;    1       0     listener
;    0       1     adressed talker
;    1       1     active talker
FLST    equ     7     ; addressed 
FTLK    equ     6     ; talker
FSDA    equ     5     ; addressed to send data
FSDI    equ     4     ; addressed to send data with PTSDI
FREN    equ     3     ; remote mode
FSER    equ     0     ; 0: PIO, 1:serial I/O is addressed
SERSP   equ     0x77     ; UART IL serial poll byte
SERRX   equ     0    ; Rx byte available
SERRXA  equ     1    ; Rx LF received
SEROVR  equ     3    ; Rx buffer overrun
SERERR  equ     4    ; Rx frame error
PTSDI   equ     0x78    ; output pointer for device ID
PTSER   equ     0x79    ; pointer for serial data
NBSER   equ     0x7a    ; number of bytes in serial buffer
;
ACCY    equ     0x7e    ; temporary accumulator
ACCX    equ     0x7f    ; temporary accumulator

; bank 2 allocation
BUFOUT  equ    0xa0   ; 16-byte output buffer (a0-af) - not used
BUFSER  equ    0xb0   ; 64-byte serial buffer (b0-ef)



;FIX1A   equ     1      ; enable HP71B HPIL 1A fix
;HSPEED  equ     1      ; high serial speed: 115k

LEDOFF   equ     4      ; LED command - RA4

DEFADDR  equ  8         ; default base address after AAU
AIDPIO   equ  0x40      ; GPIO AID
AIDSER   equ  0x42      ; serial AID


;
        org     0
        goto    Start

        org     4
Intr
; interrupt routine to sample the HPIL frame
        movwf   SAVEW       ; save W
; fast entry point for fix
Intr1        
     variable i
i=0
     while i<D'64'   ; 128 samples stored in 64 registers
        movf    CMCON,w     ; ABxx11xx
        movwf   BUF+i
        swapf   CMCON,w     ; 11xxCDxx
        andwf   BUF+i,f     ; ABxxCDxx
i+=1
     endw
        bcf     PIR1, CMIF   ; reset IT cause
        bsf     ILFLG,FRREC
        movf    SAVEW,w      ; restore W and set STATUS,Z accordingly
        retfie


GetSDI
        andlw  0x1f
        addwf  PCL,f
        dt     "PILPIO1B",0x0d,0x0a,0   ; 0;11
        dt     "PILSER1B",0x0d,0x0a,0   ; 11;11
        dt     "PIL-IO 1.1",0x0d,0x0a   ; 22;12
        dt     "(c) J-F GARNIER 2012",0x0d,0x0a,0  ; 22
; 1234567890123456789012
; PIL-IO 1.0 JFG (c)2010      

Start
        clrwdt
        movlw   B'00000000'     ; port outputs
        movwf   PORTA
        movwf   PORTB
        bsf     STATUS, RP0     ; *****************
        movlw   B'11101111'     ;make port a4 output
        movwf   TRISA           ;       /
        movlw   B'11110010'     ;make port b0,2,3 outputs
        movwf   TRISB           ;       /
        bcf     OPTION_REG,7    ; weak pull-up on port B
        call    Dly100us        ; delay 100us

; -- setup of the UART
; BRG programmed at high speed to 9.6k/19.2k/57.6k/115.2k  
;                 SPBRG code=     129 / 64  / 21  / 10    
        bcf     STATUS, RP0  ; *************************
        movlw   D'129'  
  ifdef HSPEED        
        movlw   D'10'
  endif        
        bsf     STATUS, RP0  ; *************************
        movwf   SPBRG
; setup Rx and Tx
        movlw   0x24         ; Tx enable, async high speed
        movwf   TXSTA        
        bcf     STATUS, RP0  ; *************************
        movlw   0x90         ; Serial enable, continous receive
        movwf   RCSTA        

; -- setup of the comparators
        movlw   0x1c         ;  mode 100, C1INV
        movwf   CMCON
        nop
        movf    CMCON,w       ; read

; enable INT on comparator change
        bsf     STATUS, RP0  ; *************************
        movlw   0x40            ; CMIE
        movwf   PIE1
        bcf     STATUS, RP0  ; *************************
        movlw   0x40            ; PEIE
        movwf   INTCON

; init variables        
        clrf    ILFLG
        clrf    ILSTAT
        movlw   DEFADDR
        movwf   ILADDR
        call    clrser

; make blink the LED
        call    blinkLED

        bsf     INTCON, GIE ; enable intr

        
; ======== main loop ===========
Main
; internal loop, waiting for IL frame, or serial byte
        clrwdt
        btfsc   PIR1,RCIF
        goto    inserial
        btfsc   ILFLG,FRREC
        goto    insamp
        btfsc   ILFLG,FRAV
        goto    inIL
        goto    Main
; ================================


clrpio
        bsf     STATUS, RP0     ; *****************
        movlw   B'11110010'     ;make port b0,2,3 outputs
        movwf   TRISB           ;       /
        bcf     STATUS, RP0     ; *************************
        return

clrser
        clrf    NBSER
        clrf    PTSER
        clrf    SERSP
        return


powerdn
        bcf     INTCON, GIE   ; disable intr
        sleep
        nop
        nop
        btfss   PIR1,CMIF
        goto    powerdn
        bcf     ILSTAT,FLPD      
        bsf     INTCON, GIE   ; enable intr
        return      

      
trigger
; pulse IO3 low 800us
        bsf     STATUS, RP0     ; *****************
        btfss   TRISB,7
        goto    trig9
        bcf     TRISB,7        
        call    Dly800us
        bsf     TRISB,7
trig9   bcf     STATUS, RP0     ; *****************
        return      
      
      
; -------------------------------        
; IL frame samples received.
; -------------------------------
insamp
        bcf     INTCON, GIE   ; disable intr
        bcf     PORTA,LEDOFF       ; LED on
        bcf     ILFLG,FRREC
        call    decframe      ; decode frame into FRAMEH/L,  FRAV is set if ok
        bsf     INTCON, GIE   ; enable intr
        bsf     PORTA,LEDOFF   ; LED off
        goto    Main
        
; -------------------------------        
; IL frame available.
; -------------------------------
inIL    bcf     INTCON, GIE   ; disable intr
        bcf     PORTA,LEDOFF       ; LED on
        bcf     ILFLG,FRAV
        
        ; according to frame type:
        btfss   FRAMEH,2   ; control frame?
        goto    TRDOE      ;   no (DOE), processes it
        btfsc   FRAMEH,1   ; CMD or RDY?
        goto    TRIDY      ;   no (IDY), re-transmit
        btfss   FRAMEH,0   ; RDY?
        goto    TRCMD      ;   no (CMD), processes it
        movf    FRAMEL,w   ; 
        btfss   STATUS,Z   ; RFC?
        goto    TRRDY      ;   no (RDY), process it
        ;   yes (RFC), re-transmit
        call    sendIL     ; send 
        btfsc   ILSTAT,FLPD
        call    powerdn        
        goto    Main
        
TRIDY   btfsc   ILSTAT,SSRQ
        bsf     FRAMEH,0   ; set bit SRQ in IDY frame
        
TR11    call    sendIL     ; send and return to Main
        goto    Main


        ; ------------------------------------------        
        ; frames to be processed by the application
        ; ------------------------------------------        

; -------------------------------        
; data or end frames
; -------------------------------
TRDOE
        bcf     ILSTAT,FLPD
        btfss   ILSTAT,FLST   ; addressed?
        goto    TRDOE9        ; no, return
        btfss   ILSTAT,FTLK   ; yes. active talker?
        goto    TRDOE2        ; no, listener
        ; active talker
        btfss   ILSTAT,FSDA   ; SDA or SDI on going?
        goto    TRDOE7        ; no, send EOT
        ; yes (SDA or SDI)
        btfss   ILSTAT,FSDI   ; SDI?
        goto    TRDOE1        ; no. manage SDA
        ; SDI
        ; send data from ROM using PTSDI
        movlw    HIGH GetSDI
        movwf    PCLATH
        movf    PTSDI,W
        btfsc   STATUS,Z ; PTSDI=0?
        goto    TRDOE7   ; yes, send EOT
        call    GetSDI
        incf    PTSDI
        iorlw   0
        btfss   STATUS,Z ; last byte (=0)?
        goto    TRDOE8   ; no send it
        clrf    PTSDI
        goto    TRDOE7   ; else send EOT
        
TRDOE1  ; send data from RAM using PTSER
        call    getserb
        btfsc   STATUS,Z
        goto    TRDOE7    ; no data, EOT
        goto    TRDOE8
        
TRDOE2  ; listener
        btfsc   ILSTAT,FSER
        goto    TRDOE3
        ; PIO
        movf    FRAMEL,w
        sublw   0x2f  ; 2f-w; Cy set if >=0 ie w<=2f 
        btfsc   STATUS,C
        goto    TRDOE9
        movf    FRAMEL,w
        sublw   0x40  ; 40-w; Cy set if >=0 ie w<=40 
        swapf   FRAMEL,W
        btfss   STATUS,C
        addlw   0x90   ; convert 'A'-'F' to 0xa-0xf
        xorlw   0xf0   ; invert cause a bit 1 means output (low)
        andlw   0xf0
        iorlw   2      ; keep RB0-RB3 unchanged
        bsf     STATUS, RP0     ; *****************
        movwf   TRISB
        bcf     STATUS, RP0     ; *****************
        goto    TRDOE9
TRDOE3  ; serial
        call    sndserial
        goto    TRDOE9

TRDOE7  bcf     ILSTAT,FLST  ; remains addressed talker
        bcf     ILSTAT,FSDA
        bcf     ILSTAT,FSDI
        movlw   5
        movwf   FRAMEH
        movlw   0x40    ; EOT
TRDOE8  movwf   FRAMEL
TRDOE9  goto    TR11   ; return
        
        
        
; -------------------------------        
; ready frames
; -------------------------------
TRRDY
        bcf     ILSTAT,FLPD
        btfsc   FRAMEL,7
        goto    TRAAD
        ; 
        btfss   ILSTAT,FTLK
        goto    TRSOT9
TRSOT   movf    FRAMEL,W
        sublw   0x42        ; 66  NRD ?
        btfss   STATUS,Z
        goto    TRSOT1
        bcf     ILSTAT,FSDA    ; abort transfer
        bcf     ILSTAT,FSDI
        goto    TRSOT9
TRSOT1  movf    FRAMEL,W
        sublw   0x60         ; 96  SDA
        btfss   STATUS,Z
        goto    TRSOT2
        btfss   ILSTAT,FSER
        goto    TRSOT7        ; PIO: send EOT
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        bsf     ILSTAT,FSDA
        bcf     ILSTAT,FSDI
        ; get byte from serial buffer
        call    getserb
        btfsc   STATUS,Z
        goto    TRSOT7    ; no data, EOT
        goto    TRSOT8
TRSOT2  movf    FRAMEL,W
        sublw   0x61           ; 97   SST
        btfss   STATUS,Z
        goto    TRSOT3
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        ; read PIO stat
        swapf   PORTB,W
        andlw   0x0f
        xorlw   0x0f
        btfss   ILSTAT,FSER
        goto    TRSOT8
        movf    SERSP,W     ; or get serial status byte
        bcf     SERSP,SEROVR
        bcf     SERSP,SERERR
        bcf     SERSP,SERRXA
        goto    TRSOT8
TRSOT3  movf    FRAMEL,W
        sublw   0x62             ; 98   SDI
        btfss   STATUS,Z
        goto    TRSOT4
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        bsf     ILSTAT,FSDA
        bsf     ILSTAT,FSDI
        movlw   HIGH GetSDI
        movwf   PCLATH
        clrw
        btfsc   ILSTAT,FSER
        movlw   D'11'
        movwf   PTSDI
        call    GetSDI
        incf    PTSDI
        goto    TRSOT8
TRSOT4  movf    FRAMEL,W
        sublw   0x63                  ; 99   SAI
        btfss   STATUS,Z
        goto    TRSOT9
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        movlw   AIDPIO
        btfsc   ILSTAT,FSER
        movlw   AIDSER
        goto    TRSOT8

TRSOT7  bcf     ILSTAT,FLST  ; remains addressed talker
        bcf     ILSTAT,FSDA
        movlw   5
        movwf   FRAMEH
        movlw   0x40    ; EOT
        movwf   FRAMEL
        goto    TRSOT9
TRSOT8  movwf   FRAMEL
        clrf    FRAMEH
TRSOT9  goto   TR11      ; return

TRAAD   ; AAD frames
        movf   FRAMEL,w
        sublw  0x9C  ; 9C-addr; Cy set if >=0 ie addr<=9C
        btfss  STATUS,C
        goto   TRAAD9
TRAAD1  btfsc  ILADDR,7  ; already an assigned address?
        goto   TRAAD9    ; yes
        movf   FRAMEL,w  ; no, take it as base addr
        movwf  ILADDR
        incf   FRAMEL
        incf   FRAMEL    ; takes 2 addresses
TRAAD9  goto   TR11      ; return

        
        

; -------------------------------        
; command frames
; -------------------------------
TRCMD   ; CMD frames
        bcf     ILSTAT,FLPD
        movf    FRAMEL,W
        andlw   0xe0         ; keep bits 7-5
        btfss   STATUS,Z
        goto    TRCMD1
TRCMD0  ; cmd group 0
        movf    FRAMEL,W
        sublw   1
        btfss   STATUS,Z
        goto    TRCMD01
        ; GTL
        btfsc   ILSTAT,FLST
        bcf     ILSTAT,FREN
        goto    TRCMD09
TRCMD01 movf    FRAMEL,W
        sublw   4
        btfss   STATUS,Z
        goto    TRCMD02
        ; SDC
        btfss   ILSTAT,FLST
        goto    TRCMD09
        btfss   ILSTAT,FSER
        call    clrpio
        btfsc   ILSTAT,FSER
        call    clrser
        goto    TRCMD09
TRCMD02 movf    FRAMEL,W
        sublw   8
        btfss   STATUS,Z
        goto    TRCMD03
        ; GET
        btfss   ILSTAT,FLST
        goto    TRCMD09
        btfss   ILSTAT,FSER
        call    trigger
        goto    TRCMD09
TRCMD03 movf    FRAMEL,W
        sublw   0x14
        btfss   STATUS,Z
        goto    TRCMD09
        ; DCL
        call    clrpio
        call    clrser       
TRCMD09 goto    TR11
        
TRCMD1  addlw   0xe0
        btfss   STATUS,Z
        goto    TRCMD2
        ; LAD        
        movf    FRAMEL,W
        addlw   1
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD11
        ; UNL
        btfss   ILSTAT,FTLK
        clrf    ILSTAT
        goto    TRCMD19
TRCMD11 movf    ILADDR,w
        subwf   FRAMEL,W   ; FRAMEL-ILADDR
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD12
        ; MLA (PIO)
        clrf    ILSTAT
        bsf     ILSTAT,FLST
        goto    TRCMD19
TRCMD12 addlw   -1
        btfss   STATUS,Z
        goto    TRCMD19
        ; MLA (SER)
        clrf    ILSTAT
        bsf     ILSTAT,FLST
        bsf     ILSTAT,FSER
TRCMD19 goto    TR11

TRCMD2  addlw   0xe0
        btfss   STATUS,Z
        goto    TRCMD4
        ; TAD
        ; if talker, reset talker states 
        btfsc   ILSTAT,FTLK
        clrf    ILSTAT
        ; 
        movf    ILADDR,w
        subwf   FRAMEL,W   ; FRAMEL-ILADDR
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD22
        ; MTA (PIO)
        clrf    ILSTAT
        bsf     ILSTAT,FTLK
        goto    TRCMD29
TRCMD22 addlw   -1
        btfss   STATUS,Z
        goto    TRCMD29
        ; MTA (SER)
        clrf    ILSTAT
        bsf     ILSTAT,FTLK
        bsf     ILSTAT,FSER
TRCMD29 goto    TR11

TRCMD4  addlw   0xc0
        btfss   STATUS,Z
        goto    TR11
        ; cmd group 4
        movf    FRAMEL,W
        sublw   0x90   ; IFC
        btfss   STATUS,Z
        goto    TRCMD40
        clrf    ILSTAT
        goto    TRCMD49
TRCMD40 movf    FRAMEL,W
        sublw   0x92   ; REN
        btfss   STATUS,Z
        goto    TRCMD41
        bsf     ILSTAT,FREN
        goto    TRCMD49
TRCMD41 movf    FRAMEL,W
        sublw   0x93   ; NRE
        btfss   STATUS,Z
        goto    TRCMD42
        bcf     ILSTAT,FREN
        goto    TRCMD49
TRCMD42 movf    FRAMEL,W
        sublw   0x9a   ; AAU
        btfss   STATUS,Z
        goto    TRCMD43
        movlw   DEFADDR
        movwf   ILADDR	    
        goto    TRCMD49
TRCMD43 movf    FRAMEL,W
        sublw   0x9b   ; LPD
        btfss   STATUS,Z
        goto    TRCMD49
        bsf     ILSTAT,FLPD
TRCMD49 goto    TR11
        
        
       

; -------------------------------        
; serial byte received.
; -------------------------------        
inserial
        ; ! do not desable intr! il frames can arrive anytime
        ; pay attn that Z flag is preserved during intr only if it reflects W status
        bcf     PORTA,LEDOFF      ; LED on
        call    inser
        bsf     PORTA,LEDOFF   ; LED off
        goto    Main        

inser   movf    RCREG,W
        movwf   SBYTE
        btfss   RCSTA,OERR
        goto    inSER1
; in case of USART error, reset it
        bcf     RCSTA,CREN
        nop
        bsf     RCSTA,CREN 
        bsf     SERSP,SERERR
inSER1         
        movf   NBSER,w
        addlw  1
        andlw  0x3f
        btfsc  STATUS,Z
        goto   inSER8    ; overflow, ignore byte and set SEROVR
        movwf  NBSER
        addlw  -1
        addwf  PTSER,w
        andlw  0x3f
        addlw  BUFSER                
        movwf  FSR
        movf   SBYTE,w
        movwf  INDF
        bsf    SERSP,SERRX
        sublw  0x0a
        btfsc  STATUS,Z
        bsf    SERSP,SERRXA
        goto   inSER9
inSER8  bsf    SERSP,SEROVR     ; overflow
inSER9  return
    
getserb
        movf    NBSER,W
        btfsc   STATUS,Z
        goto    getsb1    ; no data (Z set)
        decf    NBSER
        btfsc   STATUS,Z
        bcf     SERSP,SERRX  ; no more data
        movf    PTSER,w
        addlw   BUFSER
        movwf   FSR
        incf    PTSER
        bcf     PTSER,6   ; modulo 64
        movf    INDF,w
        bcf     STATUS,Z
getsb1  return


; ---------------------------------	
; send a byte from FRAMEL to serial
sndserial
        btfsc   PIR1,RCIF
        call    inser
        btfss   PIR1,TXIF
        goto    sndserial
        movf    FRAMEL,w
        movwf   TXREG
        return
        

    

; ----------------------------------------------------        
; send a frame to IL. Returns to Main as fast as possible
sendIL
        movf    CMCON,w  
        bcf     PIR1, CMIF   ; clear comparator IT
; sndframe
; send a frame from FRAME
MASKM   equ  1
MASKP   equ  8
        movf  FRAMEH,w
        movwf FRREGH
        movf  FRAMEL,w
        movwf FRREGL

        btfsc FRREGH,2
        goto  sendf1

; send '0' sync bit
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKP  ; OUT+
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKP  ; OUT+
        movwf  PORTB
        goto   sendf2

sendf1
; send '1' sync bit
        movlw  MASKP  ; OUT+
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKP  ; OUT+
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
sendf2  nop
        movlw  0  ; 
        movwf  PORTB
        nop

        movlw D'9'   ; loop for the next 9 bits
        movwf CPTBIT
                      
sendn       
        rlf   FRREGL
        rlf   FRREGH
        btfsc FRREGH,2
        goto  send1
        nop
; send '0' bit
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKP  ; OUT+
        movwf  PORTB
        goto send2

send1
; send '1' bit
        movlw  MASKP  ; OUT+
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        nop
send2   nop
        movlw  0  ; 
        movwf  PORTB

        decfsz CPTBIT
        goto   sendn

; last bit, optimized for fix with the HP71B HPIL module rev 1A
        
        nop
        rlf   FRREGL
        rlf   FRREGH
        btfsc FRREGH,2
        goto  send11
        nop
        
; send '0' bit
        movlw  MASKM  ; OUT-
        movwf  PORTB
        nop
        movlw  MASKP  ; OUT+
        goto   send21

send11
; send '1' bit
        movlw  MASKP  ; OUT+
        movwf  PORTB
        nop
        nop
        nop
        movlw  MASKM  ; OUT-
send21  movwf  PORTB
        movlw  0  ; 
        btfsc  PIR1, CMIF   ; check comparator IT
        goto   startframe   ; start of frame received!
        nop
        movwf  PORTB
                
        bsf    INTCON, GIE ; enable intr
        bsf    PORTA,LEDOFF   ; LED off        
        return

startframe	
        movwf  PORTB
        call   Intr1        ; immediately call intr (GIE set by RETFIE)
        bsf    PORTA,LEDOFF   ; LED off        
        bsf    ILFLG,FRAV   ; frame available
        return
        

                


; ---------------------------------	

decframe
; decode frame from samples in BUF (20-5f). Result in FRAME, set ILFLG,FRAV if frame found
        movlw   BUF
        movwf   FSR
        movlw   40     ; 64 registers (128 samples)
        movwf   CPTSAM
        clrf    CPTBIT
        clrf    DECFLG ; flags for decoding
; DECFLG flags: 
fBIT    equ     0    ; fBIT must be bit 0
fSTART  equ     1
fGETB   equ     2
fIDLE   equ     3
; CURIN bit positions:
fINp    equ     6
fINm    equ     7

decfra1
; get input samples in CURIN
        movf   INDF,W
        movwf  CURIN
        incf   FSR
        bcf    DECFLG,fGETB
        goto   getsam3

getsam2 
        swapf  CURIN
        bsf    DECFLG,fGETB
getsam3
; find bits
; positive pulse? 
        btfss   CURIN,fINp
        goto    fndtr1
        bsf     DECFLG,fSTART
        bcf     DECFLG,fBIT    ; last positive pulse will mean '0' bit
        bcf     DECFLG,fIDLE
        goto    fndtr4
fndtr1
; negative pulse?
        btfsc   CURIN,fINm
        goto    fndtr2
        bsf     DECFLG,fSTART
        bsf     DECFLG,fBIT    ; last negative pulse will mean '1' bit
        bcf     DECFLG,fIDLE
        goto    fndtr4
fndtr2
; check IDLE state after at least one pulse is found
        btfss   DECFLG,fSTART
        goto    fndtr4
; idle state found
        btfsc   DECFLG,fIDLE
        goto    fndtr5
        bsf     DECFLG,fIDLE
        goto    fndtr4
fndtr5
; idle found 2 times: one bit found, store it
        rrf     DECFLG,W
        rlf     FRREGL
        rlf     FRREGH
        incf    CPTBIT
        bcf     DECFLG,fSTART        
        bcf     DECFLG,fIDLE

fndtr4
        btfss  DECFLG,fGETB
        goto   getsam2

        decfsz  CPTSAM
        goto    decfra1

; end of samples. did we get exactly 11 bits?
        movf    CPTBIT,W
        sublw   D'11'
  ifdef FIX1A        
        btfsc   STATUS,Z
        goto    fndtr8              ; yes
        ; fix for HP71B HPIL module rev 1A
        ; check if start of frame was detected during sending
        btfss   ILFLG,FRAV
        goto    fndtr9  ; no
; yes. 10 bits received?        
        movf    CPTBIT,W
        sublw   D'10'
        btfss   STATUS,Z
        goto    fndtr9              ; no
; check if last sent frame was a data        
        btfsc   FRAMEH,2
        goto    fndtr9
        ; ok
        bcf     FRREGH,2            ; set 1st bit to 0
  else
        btfss   STATUS,Z
        goto    fndtr9              ; no
  endif        
        ;
fndtr8  bsf     ILFLG,FRAV          ; frame found
        movf    FRREGH,w
        andlw   7
        movwf   FRAMEH
        movf    FRREGL,w
        movwf   FRAMEL
fndtr9  return
       
 
blinkLED
; make blink the LED
        movlw   D'250'
        call    LEDon
        movlw   D'250'
        call    Delay
        
        movlw   D'250'
        call    LEDon
        movlw   D'250'
        call    Delay

        movlw   D'250'
        call    LEDon

        return   

LEDon
        movwf   CPTB
LEDon1
        clrwdt
        bcf     PORTA,LEDOFF   ; LED on
        call    Dly300us
        bsf     PORTA,LEDOFF   ; LED off
        call    Dly700us
        decfsz    CPTB,F
        goto    LEDon1
        return



; ------------------------------------------------------------------------        
        
; delay of W ms
Delay
        movwf   CPTB
d100ms  call    Dly1ms
        clrwdt
        decfsz  CPTB,F
        goto    d100ms
        return

; delay of 1 ms
Dly1ms
        call    Dly100us
        call    Dly100us
Dly800us      
        call    Dly100us
Dly700us      
        call    Dly100us
        call    Dly100us
        call    Dly100us
        call    Dly100us
Dly300us      
        call    Dly100us
        call    Dly100us
        call    Dly100us
        return

Dly100us                ; delay 100us = 125 loops of 0.8us
        movlw   D'131'  ; (from 131 to 256)
d1ms    addlw   1       ; 1 cycle (0.2 us at 20MHz)
        btfss   STATUS,Z    ; 1
        goto    d1ms        ; 2
        return


        end


